// Vektor.java - Klasa vektora sa paralelnim izracunavanjima.

class Sabirac extends Thread {
  private double a, b, c;
  public Sabirac (double aa, double bb) { a = aa; b = bb; }
  public void run () { c = a + b; }
  public double rezultat () { return c; }
}

class Mnozac extends Thread {
  private double a, b, c;
  public Mnozac (double aa, double bb) { a = aa; b = bb; }
  public void run () { c = a * b; }
  public double rezultat () { return c; }
}

class EIndeks extends Exception {
  private int ind;
  public EIndeks (int i) { ind = i; }
  public String toString ()
    { return "\n*** Nedozvoljeni indeks " + ind + " ***\n"; }
}

class EDuzina extends Exception {
  private int duz1, duz2;
  public EDuzina (int d1, int d2) { duz1 = d1; duz2 = d2; }
  public String toString ()
    { return "\n*** Neusaglasene duzine " + duz1 + " i " + duz2 + " ***\n"; }
}

class Vektor {
  private double[] niz;
  public Vektor (int duz) { niz = new double [duz]; }
  public void stavi (int ind, double broj) throws EIndeks {
    if (ind < 0 || ind >= niz.length) throw new EIndeks (ind);
    niz[ind] = broj;
  }

  public double uzmi (int ind) throws EIndeks {
    if (ind < 0 || ind >= niz.length) throw new EIndeks (ind);
    return niz[ind];
  }

  public String toString () {
    String s = "(";
    for (int i=0; i<niz.length; i++)
      { s += niz[i]; if (i < niz.length - 1) s += ","; }
    return s + ")";
  }

  public static Vektor zbir (Vektor v1, Vektor v2) throws EDuzina {
    int n = v1.niz.length, n2 = v2.niz.length;
    if (n != n2) throw new EDuzina (n, n2);
    try {
      Sabirac[] sab = new Sabirac [n];
      for (int i=0; i<n; i++)
        (sab[i] = new Sabirac (v1.uzmi(i), v2.uzmi(i))).start ();
      Vektor v = new Vektor (n);
      for (int i=0; i<n; i++) 
        try { sab[i].join(); v.stavi (i, sab[i].rezultat ()); }
          catch (InterruptedException e) {}
      return v;
    } catch (EIndeks greska) { return null; }
  }

  public static double skalpro (Vektor v1, Vektor v2) throws EDuzina {
    int n = v1.niz.length, n2 = v2.niz.length;
    if (n != n2) throw new EDuzina (n, n2);
    try {
      Mnozac[] mno = new Mnozac [n];
      for (int i=0; i<n; i++)
        (mno[i] = new Mnozac (v1.uzmi(i), v2.uzmi(i))).start ();
      double s = 0;
      for (int i=0; i<n; i++) 
        try { mno[i].join(); s += mno[i].rezultat (); }
          catch (InterruptedException e) {}
      return s;
    } catch (EIndeks greska) { return 0; }
  }

}  